//  Table for Fm_T 
#ifndef TABLE_FOR_FM_T_H
#define TABLE_FOR_FM_T_H

namespace PInt {

//double cal_Fm_T(int m,double T);


}

#include "detail/Table_Fm_T_detail.hpp"

#endif // end of ifndef TABLE_FOR_FM_T_H
